unit DelphiFour;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics,
  Controls, Forms, Dialogs, ExptIntf, ExtCtrls;

type
  TFunDelphiFour = class(TIExpert)
    function GetName: string; override;
    function GetStyle: TExpertStyle; override;
    function GetState: TExpertState; override;
    function GetIDString: string; override;
    function GetMenuText: string; override;
    procedure Execute; override;
    // custom methods
    procedure ChangeTab (Sender: TObject);
    procedure Explode (Sender: TObject);
    procedure BugFix (Sender: TObject);
    procedure About (Sender: TObject);
    procedure TimerCall1 (Sender: TObject);
    procedure TimerCall2 (Sender: TObject);
    procedure OiFont (Sender: TObject);
  end;

procedure Register;

implementation

uses
  ComCtrls, Menus, ShellApi, StdCtrls, MMSystem;

var
  d4: boolean = false;

function TFunDelphiFour.GetName: string;
begin
  Result := 'FunSide4 Wizard';
end;

function TFunDelphiFour.GetStyle: TExpertStyle;
begin
  Result := esStandard;
end;

function TFunDelphiFour.GetState: TExpertState;
begin
  if not d4 then
    Result := [esEnabled]
  else
    Result := [esChecked];
end;

function TFunDelphiFour.GetIDString: string;
begin
  Result := 'marco.FunDelphi4.Wizard';
end;

function TFunDelphiFour.GetMenuText: string;
begin
  Result := 'Fun Delphi 4 Wizard!';
end;

procedure TFunDelphiFour.ChangeTab (Sender: TObject);
begin
  with Application.MainForm.FindComponent ('TabControl') as TTabControl do
  begin
    TabIndex := Tabs.IndexOf (
      (Sender as TMenuItem).Caption);
    OnChange (self);
  end;
end;

{$r d4.res}

procedure TFunDelphiFour.Execute;
var
  I: Integer;
  m: TMenuItem;
begin
  if not d4 then
  begin
    d4 := true;
    Application.Title := 'Delphi 4';
    Application.Icon.Handle :=
      LoadIcon (HInstance, 'D4');
    with Application.MainForm.
      FindComponent ('TabControl') as TTabControl do
    begin
      MultiLine := True;
      Height := Height + 18;
      SendMessage (
        Application.MainForm.Handle,
        wm_Size, 0, 0);
      if PopupMenu.Items.Count > 5 then
        for I := PopupMenu.Items.Count - 1 downto 5 do
          PopupMenu.items[i].Free;
      // separator
      m := TMenuItem.Create (Application);
      m.Caption := '-';
      PopupMenu.Items.Add (m);

      // Patented code...
      for I := 0 to Tabs.Count - 1 do
      begin
        m := TMenuItem.Create (Application);
        m.Caption := Tabs [I];
        m.OnClick := ChangeTab;
        PopupMenu.Items.Add (m);
      end;
      // separator
      m := TMenuItem.Create (Application);
      m.Caption := '-';
      PopupMenu.Items.Add (m);
      // exploder
      m := TMenuItem.Create (Application);
      m.Caption := 'Extra Cool SECRET feature';
      m.OnClick := Explode;
      PopupMenu.Items.Add (M);
      // fixer
      m := TMenuItem.Create (Application);
      m.Caption := 'Bug Fix';
      m.OnClick := BugFix;
      PopupMenu.Items.Add (M);
      // font sizer
      m := TMenuItem.Create (Application);
      m.Caption := 'Inspector Font...';
      m.OnClick := OiFont;
      PopupMenu.Items.Add (M);
    end;
    with Application.MainForm.FindComponent (
      'HelpAboutItem') as TMenuItem do
    begin
      Caption := 'About Delphi 4 Preview...';
      OnClick := About;
    end;
  end;
end;

type PtsType = array [0..15, 0..1] of Integer;
const Pts: PtsType =
  ((0, 0), (800, 0), (800, 200),
   (400, 200), (400, 120), (500, 180),
   (465, 105), (550, 17), (405, 70),
   (320, -10), (380, 140), (250, 250), (400, 120),
   (400, 200), (0, 200), (0, 0));

procedure TFunDelphiFour.Explode(Sender: TObject);
var
  HRegion1: THandle;
begin
  HRegion1 := CreatePolygonRgn (Pts,
    sizeof (Pts) div 8,
    alternate);
  PlaySound ('c:\conferences\funside\boom.wav', 0, snd_sync);
  SetWindowRgn (
    Application.MainForm.Handle,
    HRegion1, True);
end;

procedure TFunDelphiFour.BugFix(Sender: TObject);
begin
  SetWindowRgn (
    Application.MainForm.Handle,
    0, True);
end;

procedure TFunDelphiFour.OiFont(Sender: TObject);
var
  Fd: TFontDialog;
  Oi: TForm;
  I: Integer;
begin
  fd := TFontDialog.Create (nil);
  oi := nil;
  try
    for I := 0 to Screen.FormCount - 1 do
      if Screen.Forms[I].ClassName = 'TPropertyInspector' then
        oi := Screen.Forms[I];
    if not Assigned (oi) then
      raise Exception.Create ('oi not found');
    Fd.Font := oi.Font;
    if fd.Execute then
      oi.Font := Fd.Font;
  finally
    fd.Free;
  end;
end;

procedure TFunDelphiFour.About(Sender: TObject);
var
  ObjDelphi: TWinControl;
  Meth: TMethod;
  Evt: TNotifyEvent;
  P: Pointer;
  Timer1: TTimer;
begin
  Timer1 := TTimer.Create (nil);
  Timer1.Interval := 3000;
  Timer1.OnTimer := TimerCall1;
  try
    // shell about MS
    ShellAbout (Application.MainForm.Handle,
      PChar ('Visual Basic 10 Preview'),
      PChar ('To be released in year 2010'),
      Application.Icon.Handle);

    Timer1.OnTimer := TimerCall2;
    Timer1.Enabled := True;

    // original method call
    ObjDelphi := Application.MainForm;
    if ObjDelphi <> nil then
    begin
      P := ObjDelphi.MethodAddress ('HelpAbout');
      Meth.Code := P;
      Meth.Data := ObjDelphi;
      Evt := TNotifyEvent (Meth);
      Evt (ObjDelphi);
    end
    else
      ShowMessage ('AppBuilder object not accessible');
  finally
    Timer1.Free;
  end;
end;

procedure TFunDelphiFour.TimerCall1(Sender: TObject);
var
  hdcDesk: THandle;
  Pen: TPen;
begin
  hdcDesk := GetWindowDC (GetDesktopWindow);
  Pen := TPen.Create;
  Pen.Color := clRed;
  Pen.Width := 30;
  SelectObject (hdcDesk, Pen.Handle);
  MoveToEx (hdcDesk, 50, 100, nil);
  LineTo (hdcDesk, 350, 300);
  MoveToEx (hdcDesk, 50, 300, nil);
  LineTo (hdcDesk, 350, 100);
  ReleaseDC (hdcDesk, GetDesktopWindow);
  Pen.Free;
  (Sender as TTimer).Enabled := False;
end;

procedure TFunDelphiFour.TimerCall2(Sender: TObject);
begin
  with Screen.Forms[0].Components[5] as TLabel do
    Caption := 'InterGalactical Client/Server Suite';
  with Screen.Forms[0].Components[7] as TLabel do
    Caption := 'Version 4.0';
  with Screen.Forms[0].Components[4] as TLabel do
    Caption := 'Build 09.09.97 (For Your Eyes Only)';
  with Screen.Forms[0].Components[2] as TLabel do
    Caption := 'Copyright 1983-1998 Borland Stellar Inc.';
  (Sender as TTimer).Enabled := False;
end;

procedure Register;
begin
  RegisterLibraryExpert (TFunDelphiFour.Create);
end;

end.
