unit ExpGame;

interface

uses
  Windows,
  Dialogs,
  Classes,
  SysUtils,
  ExptIntf,
  ToolIntf,
  VirtIntf,
  FileCtrl,
  Controls,
  ExpGForm,
  IStreams,
  ExpGResF,
  Forms;

type
  TGameExpert = class (TIExpert)
  public
    function GetStyle: TExpertStyle; override;
    function GetName: string; override;
    function GetComment: string; override;
    function GetGlyph: HBITMAP; override;
    function GetState: TExpertState; override;
    function GetIDString: string; override;
    function GetMenuText: string; override;
    procedure Execute; override;
  end;

  procedure Register;

implementation

function TGameExpert.GetStyle: TExpertStyle;
begin
  Result := esStandard;
end;

function TGameExpert.GetName: String;
begin
  Result := 'Game Expert'
end;

function TGameExpert.GetComment: String;
begin
  Result := '';  // no thanks
end;

function TGameExpert.GetGlyph: HBITMAP;
begin
  Result := 0;  // no thanks
end;

function TGameExpert.GetState: TExpertState;
begin
  Result := [esEnabled];
end;

function TGameExpert.GetIDString: String;
begin
  Result := 'MarcoCantu.GameExpert'
end;

function TGameExpert.GetMenuText: String;
begin
  Result := '&Game Expert...'
end;

procedure TGameExpert.Execute;
var
  Buffer: Array[1..4096] of Char;
  UnitName, FormName: string;
  FileStream: TFileStream;
  MemoryStream: TMemoryStream;
  IUnitStream,IFormStream: TIMemoryStream;
  ProjectDir: string;
begin
  SetLength (UnitName, 20);
  SetLength (FormName, 20);
  ToolServices.GetNewModuleName (
    UnitName, FormName);
  ProjectDir := ExtractFilePath (FormName);
  if ProjectDir[Length(ProjectDir)] = '\' then
    Delete(ProjectDir,Length(ProjectDir),1);
  ChDir(ProjectDir);
  UnitName := ProjectDir + '\' +
    ChangeFileExt(UnitName,'');

  GameExpertForm :=
    TGameExpertForm.Create (Application);
  GameExpertForm.UnitName := UnitName;
  UnitName := UnitName + '.PAS';

  if (GameExpertForm.ShowModal = idOk) then
  begin
    FormName := ProjectDir + '\' +
      ExtractFileName (ChangeFileExt (FormName, '.DFM'));

    WriteComponentResFile (FormName, ResultForm);
    ResultForm.Free;

    MemoryStream := TMemoryStream.Create;
    FileStream := TFileStream.Create (
      FormName, fmShareDenyNone);
    repeat until MemoryStream.Write (
      Buffer,FileStream.Read (Buffer, 4096)) = 0;
    FileStream.Free;
    MemoryStream.Position := 0;
    IFormStream := TIMemoryStream.Create (MemoryStream);
    IFormStream.OwnStream := True;

    MemoryStream := TMemoryStream.Create;
    FileStream := TFileStream.Create (
      UnitName, fmShareDenyNone);
    repeat until MemoryStream.Write(
      Buffer,FileStream.Read (Buffer, 4096)) = 0;
    FileStream.Free;
    MemoryStream.Position := 0;
    IUnitStream := TIMemoryStream.Create (MemoryStream);
    IUnitStream.OwnStream := True;

    ToolServices.CreateModule (
      UnitName, IUnitStream, IFormStream,
      [cmAddToProject, cmShowSource, cmShowForm]);

    GameExpertForm.Free;
  end;
end;

procedure Register;
begin
  RegisterLibraryExpert(TGameExpert.Create)
end;

end.
