unit autofont;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls;

type
  TAutoFont = class(TComponent)
  private
    FTimer: TTimer;
    FInterval: Cardinal;
    FFixedSize, FAllAlike: Boolean;
  protected
    procedure OnTimer (Sender: TObject);
    procedure SetInterval (Value: Cardinal);
  public
    constructor Create (AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Interval: Cardinal
      read FInterval write SetInterval default 10000;
    property FixedSize: Boolean
      read FFixedSize write FFixedSize default True;
    property AllAlike: Boolean
      read FAllAlike write FAllAlike default True;
  end;

procedure Register;

implementation

constructor TAutoFont.Create (AOwner: TComponent);
begin
  inherited Create (AOwner);
  FTimer := TTimer.Create (AOwner);
  FTimer.Interval := 10000;
  FInterval := 10000;
  FTimer.Enabled := True;
  FTimer.OnTimer := OnTimer;
  FAllAlike := True;
  FFixedSize := True;
end;

destructor TAutoFont.Destroy;
begin
  FTimer.Enabled := False;
  FTimer.Free;
  inherited Destroy;
end;

procedure TAutoFont.OnTimer (Sender: TObject);
var
  I: Integer;
  Fnt: TFont;
begin
  (Owner as TForm).Font.Name :=
    Screen.Fonts [Random (Screen.Fonts.Count)];
  if not FFixedSize then
    (Owner as TForm).Font.Size := Random (36);
  if not FAllAlike then
  begin
    Fnt := TFont.Create;
    Fnt.Assign ((Owner as TForm).Font);
    for I := 0 to Owner.ComponentCount - 1 do
    begin
      Fnt.Name := Screen.Fonts [Random (Screen.Fonts.Count)];
      if Owner.Components [I] is TWinControl then
        SendMessage (TWinControl (Owner.Components [I]).Handle,
          wm_SetFont, Fnt.Handle, MakeLong (1,0));
    end;
    Fnt.Free;
  end;
end;

procedure TAutoFont.SetInterval (Value: Cardinal);
begin
  if Value <> FInterval then
  begin
    FInterval := Value;
    FTimer.Interval := Value;
  end;
end;

procedure Register;
begin
  RegisterComponents('FunSide', [TAutoFont]);
end;

initialization
  RegisterClasses ([TTimer]);
  Randomize;
end.
